"""***********************************************************************************
Example showing how to connect to an iPad running Breeze Booth for iPad via Bluetooth
and control NeoPixel LED strips
------------------------------------------------------------------------------------
Requirements:
Adafruit ItsyBitsy nRF52840 Express - Bluetooth LE - https://www.adafruit.com/product/4481
or Adafruit Feather nRF52840 Express - https://www.adafruit.com/product/4062

Information on setting up CircuitPython:
https://learn.adafruit.com/adafruit-itsybitsy-nrf52840-express/circuitpython

Information on using Adafruit's LED animation module:
https://learn.adafruit.com/circuitpython-led-animations

IMPORTANT
Please see Adafruit's tutorials for information on powering LED strips
otherwise you risk damaging the microcontroller board:
https://learn.adafruit.com/adafruit-neopixel-uberguide

This code comes with no warranty or support whatsoever and may be freely copied or
modified as required.

Written by Chris Breeze, Breeze Systems Limited, https://breezesys.com
20210311: Initial version
20220219: Updated to use latest CircuitPython libraries
************************************************************************************"""
import board
import time
import binascii
from adafruit_ble import BLERadio
from adafruit_ble.advertising.standard import ProvideServicesAdvertisement
from adafruit_ble.services.nordic import UARTService

import neopixel
import adafruit_fancyled.adafruit_fancyled as fancy
from adafruit_led_animation.animation import Animation
from adafruit_led_animation.color import PURPLE, WHITE, BLACK, AMBER, JADE, MAGENTA, ORANGE, BLUE, RED, GREEN, CYAN
from adafruit_led_animation.animation.solid import Solid
from adafruit_led_animation.animation.blink import Blink
from adafruit_led_animation.animation.sparkle import Sparkle

# BIGHTNESS is set in the range 0 to 1 where 1 is the brightest
BRIGHTNESS = 0.1

# pin connected to the NeoPixel data (D5 on the Adafruit ItsyBitsy nRF52840 Express - Bluetooth LE is
# a special 5V logic level suitable for driving NeoPixels)
pixel_pin = board.D5

# the number of LEDs in the strip
pixel_num = 144

# the pixel type of the LED strip
# this example assumes an SK6812 RGB+W LED strip is being used (the separate white LED in each pixel gives whites with better
# color than using a RGB LED strip)
pixel_type = neopixel.GRBW

# LED animation class to rotate the LEDs one pixel at a time to the left or right
class Rotator(Animation):
    def __init__(self, pixel_object, speed, direction = 0, name=None):
        super().__init__(pixel_object, speed, None, name=name)
        self._direction = direction

    def draw(self):
        n = len(self.pixel_object) - 1
        if self._direction <= 0:
            temp = self.pixel_object[0]
            self.pixel_object[0:n] = self.pixel_object[1:]
            self.pixel_object[n] = temp
        else:
            temp = self.pixel_object[n]
            self.pixel_object[1:] = self.pixel_object[0:n]
            self.pixel_object[0] = temp

# LED animation class to wipe a color
class Wipe(Animation):
    def __init__(self, pixel_object, speed, color, name=None, wipers=1):
        super().__init__(pixel_object, speed, color, name=name)
        self._index = 0
        self.wipers = wipers if wipers > 0 else 1

    def draw(self):
        if self._index < len(self.pixel_object):
            length = len(self.pixel_object) // self.wipers
            for i in range(self.wipers):
                offset = (i * length + self._index) % len(self.pixel_object)
                self.pixel_object[offset] = self.color
            self._index += 1

    def reset(self):
        self._index = 0

# helper fonction to load a rainbow into an LED strip
def rainbow(pixels):
    n = len(pixels)
    for i in range(n):
        pixels[i] = fancy.gamma_adjust(fancy.CHSV(i / n)).pack()

# helper function to load a series of colored dashes
def dashes(pixels, color1, dashLength1, color2, dashLength2):
    for i in range(len(pixels)):
        index = i % (dashLength1 + dashLength2)
        pixels[i] = color1 if index < dashLength1 else color2

# set up the device as a Bluetooth peripheral providing the Nordic UART service
ble = BLERadio()
ble.name = "BreezeLED_" + binascii.hexlify(ble.address_bytes).decode()
uart = UARTService()
advertisement = ProvideServicesAdvertisement(uart)

# setup the NeoPixel strip and some LED animations
pixels = neopixel.NeoPixel(pixel_pin, pixel_num, brightness=BRIGHTNESS, auto_write=False, pixel_order=pixel_type)
blink = Blink(pixels, speed=1, color=BLUE)
wipe = Wipe(pixels, speed=0.1, color=JADE, wipers=8)
rotator = Rotator(pixels, 0.025)
sparkle = Sparkle(pixels, speed=0.05, color=AMBER, num_sparkles=10)
solid = Solid(pixels, color=(255,255,255,255))

while True:
    # advertise the service, display blinking blue LEDs and wait for a connection
    ble.start_advertising(advertisement)
    blink.colors = [BLUE, BLACK]
    animation = blink
    while not ble.connected:
        # run the LED animation
        animation.animate()

    # connected: read "s:" status messages from the iPad and run a different LED animation for each screen
    wipe.reset()
    wipe.wipers = 36
    wipe.color = JADE
    animation = wipe
    while ble.connected:
        # run the LED animation
        animation.animate()
        # then poll the UART for new data
        if uart.in_waiting:
            s = uart.readline().decode("UTF-8").strip()
            if s[0:2] == "s:":
                # read the screen status and select a suitable LED animation to display
                print("Screen:", s[2:])
                if "App settings" in s:
                    wipe.reset()
                    wipe.color = ORANGE
                    wipe.wipers = 18
                    wipe.speed = 0.1
                    animation = wipe
                elif "Startup" in s:
                    wipe.reset()
                    wipe.wipers = 18
                    wipe.speed = 0.1
                    wipe.color = JADE
                    animation = wipe
                elif "Ready" in s:
                    rainbow(pixels)
                    rotator.speed = 0.1
                    animation = rotator
                elif "Countdown" in s:
                    wipe.reset()
                    wipe.wipers = 1
                    wipe.speed = 0.01
                    wipe.color = (255,255,255)
                    animation = wipe
                elif "Capture" in s:
                    solid.color = (255,255,255,255)
                    animation = solid
                elif "TakingPhoto" in s:
                    solid.color = (255,255,255,255)
                    animation = solid
                elif "Preview" in s:
                    solid.color = (0,20,0)
                    animation = solid
                elif "Processing" in s:
                    dashes(pixels, BLUE, 5, RED, 4)
                    rotator.speed = 0.2
                    animation = rotator
                elif "Share" in s:
                    animation = sparkle
